#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "include/microSynth.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    audioOutMS.closeStream();
    midiInMS->closePort();
    //midiOutMS->closePort();
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::on_WaveLFO1_currentIndexChanged(int index)
{
    synth->setLFO1wave(index);
}

void MainWindow::on_LFOOSC1Vol_currentIndexChanged(int index)
{
    if (index == 0)
    {
        synth->isLFO1toVO1(false);
        synth->isLFO2toVO1(false);
    }

    if (index == 1)
    {
        synth->isLFO1toVO1(true);
        synth->isLFO2toVO1(false);
    }

    if (index == 2)
    {
        synth->isLFO1toVO1(false);
        synth->isLFO2toVO1(true);
    }
}

void MainWindow::on_actionExit_triggered()
{
    this->~MainWindow();
    exit(0);
}

void MainWindow::on_startEng_clicked(bool checked)
{
    if (checked)
    {
        if (!audioOutMS.isStreamOpen())
        {
            try {setAudio();}
            catch (... ) {std::cerr << "No device driver available. Free your ALSA ports or start Jack engine.\n";}
        }
        else
        {
            audioOutMS.closeStream();
            setAudio();
        }
    }
    else
    {
        if (audioOutMS.isStreamOpen())
            audioOutMS.closeStream();
        else
        {};
    }
}

void MainWindow::on_WaveOSC1_currentIndexChanged(int index)
{
    synth->setOSC1wave(index);
}

void MainWindow::on_WaveOSC2_currentIndexChanged(int index)
{
    synth->setOSC2wave(index);
}

void MainWindow::on_WaveOSC3_currentIndexChanged(int index)
{
    synth->setOSC3wave(index);
}

void MainWindow::on_Amp_valueChanged(int value)
{
    synth->setAmp(value);
}

void MainWindow::on_Pan_valueChanged(int value)
{
    synth->setPan(value);
}

void MainWindow::on_WaveLFO2_currentIndexChanged(int index)
{
    synth->setLFO2wave(index);
}

void MainWindow::on_volOSC1_valueChanged(int value)
{
    synth->setVolOSC1(value);
}

void MainWindow::on_panOSC1_valueChanged(int value)
{
    synth->setPanOSC1(value);
}

void MainWindow::on_volOSC2_valueChanged(int value)
{
    synth->setVolOSC2(value);
}

void MainWindow::on_panOSC2_valueChanged(int value)
{
    synth->setPanOSC2(value);
}

void MainWindow::on_volOSC3_valueChanged(int value)
{
    synth->setVolOSC3(value);
}

void MainWindow::on_panOSC3_valueChanged(int value)
{
    synth->setPanOSC3(value);
}

void MainWindow::on_pushButton_clicked(bool checked)
{
    if (checked)
    {
        if (audioOutMS.isStreamOpen())
            synth->setControlVol(100);
        else
        {};
    }
    else
    {
        if (audioOutMS.isStreamOpen())
            synth->setControlVol(0);
        else
        {};
    }
}

void MainWindow::on_freqLFO1_valueChanged(int value)
{
    synth->setFreqLFO1(value);
}

void MainWindow::on_volLFO1_valueChanged(int value)
{
    synth->setVolLFO1(value);
}

void MainWindow::on_freqLFO2_valueChanged(int value)
{
    synth->setFreqLFO2(value);
}

void MainWindow::on_volLFO2_valueChanged(int value)
{
    synth->setVolLFO2(value);
}

void MainWindow::on_LFOOSC2Vol_currentIndexChanged(int index)
{
    if (index == 0)
    {
        synth->isLFO1toVO2(false);
        synth->isLFO2toVO2(false);
    }

    if (index == 1)
    {
        synth->isLFO1toVO2(true);
        synth->isLFO2toVO2(false);
    }

    if (index == 2)
    {
        synth->isLFO1toVO2(false);
        synth->isLFO2toVO2(true);
    }
}

void MainWindow::on_LFOOSC3Vol_currentIndexChanged(int index)
{
    if (index == 0)
    {
        synth->isLFO1toVO3(false);
        synth->isLFO2toVO3(false);
    }

    if (index == 1)
    {
        synth->isLFO1toVO3(true);
        synth->isLFO2toVO3(false);
    }

    if (index == 2)
    {
        synth->isLFO1toVO3(false);
        synth->isLFO2toVO3(true);
    }
}

void MainWindow::on_LFOOSC1Pan_currentIndexChanged(int index)
{
    if (index == 0)
    {
        synth->isLFO1toPO1(false);
        synth->isLFO2toPO1(false);
    }

    if (index == 1)
    {
        synth->isLFO1toPO1(true);
        synth->isLFO2toPO1(false);
    }

    if (index == 2)
    {
        synth->isLFO1toPO1(false);
        synth->isLFO2toPO1(true);
    }
}

void MainWindow::on_LFOOSC2Pan_currentIndexChanged(int index)
{
    if (index == 0)
    {
        synth->isLFO1toPO2(false);
        synth->isLFO2toPO2(false);
    }

    if (index == 1)
    {
        synth->isLFO1toPO2(true);
        synth->isLFO2toPO2(false);
    }

    if (index == 2)
    {
        synth->isLFO1toPO2(false);
        synth->isLFO2toPO2(true);
    }
}

void MainWindow::on_LFOOSC3Pan_currentIndexChanged(int index)
{
    if (index == 0)
    {
        synth->isLFO1toPO3(false);
        synth->isLFO2toPO3(false);
    }

    if (index == 1)
    {
        synth->isLFO1toPO3(true);
        synth->isLFO2toPO3(false);
    }

    if (index == 2)
    {
        synth->isLFO1toPO3(false);
        synth->isLFO2toPO3(true);
    }
}

void MainWindow::on_checkBox_clicked(bool checked)
{
    if (checked) synth->setAuralValue(true);
    else synth->setAuralValue(false);
}

void MainWindow::on_sratesel_currentIndexChanged(int index)
{
    switch (index) {
    case 0:
        synth->setSampleRate(22050);
        break;
    case 1:
        synth->setSampleRate(44100);
        break;
    case 2:
        synth->setSampleRate(48000);
        break;
    case 3:
        synth->setSampleRate(88200);
        break;
    case 4:
        synth->setSampleRate(96000);
        break;
    case 5:
        synth->setSampleRate(192000);
        break;
    default:
        synth->setSampleRate(44100);
    }
}
